/**
 * A library for compiling and running TI-Basic.
 *
 * @packageDocumentation
 */

/// <reference types="jquery" />

/**
 * @alpha
 */
declare const ANS = "ans";

/**
 * @alpha
 */
declare interface Ans {
    type: typeof ANS;
}

/**
 * @alpha
 */
declare type Assignable = Variable | ListIndex;

/**
 * @alpha
 */
declare const AssignmentStatement = "assign";

/**
 * @alpha
 */
declare interface AssignmentStatement {
    type: typeof AssignmentStatement;
    source?: string;
    value: ValueExpression;
    assignable: Assignable;
}

/**
 * @alpha
 */
declare const BINARY = "binary";

/**
 * @alpha
 */
declare interface BinaryExpression {
    type: typeof BINARY;
    operator: string;
    left: ValueExpression;
    right: ValueExpression;
}

/**
 * @alpha
 */
declare const ClrHome = "ClrHome";

/**
 * @alpha
 */
declare interface ClrHome {
    type: typeof ClrHome;
    source?: string;
}

/**
 * @alpha
 */
declare const ClrTable = "ClrTable";

/**
 * @alpha
 */
declare interface ClrTable {
    type: typeof ClrTable;
    source?: string;
}

/**
 * @alpha
 */
declare type CtlStatement = IfStatement | ThenStatement | ElseStatement | ForLoop | WhileLoop | RepeatLoop | EndStatement | PauseStatement | LabelStatement | GotoStatement | IncrementSkip | DecrementSkip | MenuStatement | ProgramStatement | ReturnStatement | StopStatement | DelVarStatement | GraphStyleStatement | OpenLibStatement | ExecLibStatement;

/**
 * @alpha
 */
declare const DecrementSkip = "DecrementSkip";

/**
 * @alpha
 */
declare interface DecrementSkip {
    type: typeof DecrementSkip;
    source?: string;
    variable: Variable | null;
    end: ValueExpression | null;
}

/**
 * @alpha
 */
declare const DelVarStatement = "DelVarStatement";

/**
 * @alpha
 */
declare interface DelVarStatement {
    type: typeof DelVarStatement;
    source?: string;
    variable: Variable | null;
}

/**
 * @alpha
 */
declare const DispGraph = "DispGraph";

/**
 * @alpha
 */
declare interface DispGraph {
    type: typeof DispGraph;
    source?: string;
}

/**
 * @alpha
 */
declare const Display = "display";

/**
 * @alpha
 */
declare interface Display {
    type: typeof Display;
    source?: string;
    value: ValueExpression | null;
}

/**
 * @alpha
 */
declare const DispTable = "DispTable";

/**
 * @alpha
 */
declare interface DispTable {
    type: typeof DispTable;
    source?: string;
}

/**
 * @alpha
 */
declare const ElseStatement = "ElseStatement";

/**
 * @alpha
 */
declare interface ElseStatement {
    type: typeof ElseStatement;
    source?: string;
    extra: boolean | null;
}

/**
 * @alpha
 */
declare const EmptyStatement = "empty";

/**
 * @alpha
 */
declare interface EmptyStatement {
    type: typeof EmptyStatement;
    source?: string;
}

/**
 * @alpha
 */
declare const EndStatement = "EndStatement";

/**
 * @alpha
 */
declare interface EndStatement {
    type: typeof EndStatement;
    source?: string;
    extra: boolean | null;
}

/**
 * @alpha
 */
export declare function exec(source: string, callback: (output: string) => void): void;

/**
 * @alpha
 */
declare const ExecLibStatement = "ExecLibStatement";

/**
 * @alpha
 */
declare interface ExecLibStatement {
    type: typeof ExecLibStatement;
    source?: string;
    name: string;
}

/**
 * @alpha
 */
declare const ForLoop = "ForLoop";

/**
 * @alpha
 */
declare interface ForLoop {
    type: typeof ForLoop;
    source?: string;
    variable: Variable | null;
    start: ValueExpression | null;
    end: ValueExpression | null;
    step: ValueExpression | null;
    args: boolean;
    extra: boolean | null;
}

/**
 * @alpha
 */
declare const Get = "Get";

/**
 * @alpha
 */
declare interface Get {
    type: typeof Get;
    source?: string;
    variable: Variable | null;
}

/**
 * @alpha
 */
declare const GetCalc = "GetCalc";

/**
 * @alpha
 */
declare interface GetCalc {
    type: typeof GetCalc;
    source?: string;
    variable: Variable | null;
    portflag: ValueExpression | null;
}

/**
 * @alpha
 */
declare const GetKey = "GetKey";

/**
 * @alpha
 */
declare interface GetKey {
    type: typeof GetKey;
}

/**
 * @alpha
 */
declare const GotoStatement = "GotoStatement";

/**
 * @alpha
 */
declare interface GotoStatement {
    type: typeof GotoStatement;
    source?: string;
    location: string;
}

/**
 * @alpha
 */
declare const GraphStyleStatement = "GraphStyleStatement";

/**
 * @alpha
 */
declare interface GraphStyleStatement {
    type: typeof GraphStyleStatement;
    source?: string;
    equation: ValueExpression | null;
    style: ValueExpression | null;
}

/**
 * @alpha
 */
declare const IfStatement = "IfStatement";

/**
 * @alpha
 */
declare interface IfStatement {
    type: typeof IfStatement;
    source?: string;
    value: ValueExpression | null;
    extra: boolean | null;
}

/**
 * @alpha
 */
declare const IncrementSkip = "IncrementSkip";

/**
 * @alpha
 */
declare interface IncrementSkip {
    type: typeof IncrementSkip;
    source?: string;
    variable: Variable | null;
    end: ValueExpression | null;
}

/**
 * @alpha
 */
declare const Input = "Input";

/**
 * @alpha
 */
declare interface Input {
    type: typeof Input;
    source?: string;
    text: ValueExpression | null;
    variable: Variable | null;
}

/**
 * @alpha
 */
declare type IoStatement = Input | Prompt | Display | DispGraph | DispTable | Output | ClrHome | ClrTable | GetCalc | Get | Send;

/**
 * @alpha
 */
declare const LabelStatement = "LabelStatement";

/**
 * @alpha
 */
declare interface LabelStatement {
    type: typeof LabelStatement;
    source?: string;
    location: string;
}

/**
 * @alpha
 */
declare const lib = "lib";

/**
 * @alpha
 */
declare const LIST = "list";

/**
 * @alpha
 */
declare const LISTINDEX = "listIndex";

/**
 * @alpha
 */
declare interface ListIndex {
    type: 'listIndex';
    list: ListVariable;
    index: ValueExpression;
}

/**
 * @alpha
 */
declare interface ListLiteral {
    type: typeof LIST;
    resolved: false;
    elements: ValueExpression[];
}

/**
 * @alpha
 */
declare interface ListResolved {
    type: typeof LIST;
    resolved: true;
    elements: NumberResolved[];
}

/**
 * @alpha
 */
declare const LISTVARIABLE = "listVariable";

/**
 * @alpha
 */
declare interface ListVariable {
    type: 'listVariable';
    name: string;
    custom: boolean;
}

/**
 * @alpha
 */
declare const ListVariablePrefix = "List";

/**
 * @alpha
 */
declare interface MenuChoice {
    option: TiString;
    location: string;
}

/**
 * @alpha
 */
declare const MenuStatement = "MenuStatement";

/**
 * @alpha
 */
declare interface MenuStatement {
    type: typeof MenuStatement;
    source?: string;
    title: ValueExpression | null;
    choices: MenuChoice[];
}

/**
 * @alpha
 */
declare const NUMBER = "number";

/**
 * @alpha
 */
declare interface NumberLiteral {
    type: typeof NUMBER;
    resolved: false;
    integer?: number;
    fraction?: number | null;
    exponent?: number | null;
}

/**
 * @alpha
 */
declare interface NumberResolved {
    type: typeof NUMBER;
    resolved: true;
    float: number;
}

/**
 * @alpha
 */
declare interface NumericVariable {
    type: 'variable';
    name: string;
}

/**
 * @alpha
 */
export declare function on(type: string, listener: () => void): void;

/**
 * @alpha
 */
declare const OpenLibStatement = "OpenLibStatement";

/**
 * @alpha
 */
declare interface OpenLibStatement {
    type: typeof OpenLibStatement;
    source?: string;
    name: string;
}

/**
 * @alpha
 */
declare const Output = "Output";

/**
 * @alpha
 */
declare interface Output {
    type: typeof Output;
    source?: string;
    row: ValueExpression | null;
    column: ValueExpression | null;
    value: ValueExpression | null;
}

/**
 * @alpha
 */
export declare function parse(source: string, options?: ParseOptions): types.Statement[];

/**
 * @alpha
 */
export declare interface ParseOptions {
    sourceMap?: string;
}

/**
 * @alpha
 */
declare const PauseStatement = "PauseStatement";

/**
 * @alpha
 */
declare interface PauseStatement {
    type: typeof PauseStatement;
    source?: string;
}

/**
 * @alpha
 */
export declare interface ProgramHandle {
    getStatus(): string;
    isActive(): boolean;
    stop(): void;
}

/**
 * @alpha
 */
declare const ProgramStatement = "ProgramStatement";

/**
 * @alpha
 */
declare interface ProgramStatement {
    type: typeof ProgramStatement;
    source?: string;
    name: string;
}

/**
 * @alpha
 */
declare const Prompt = "Prompt";

/**
 * @alpha
 */
declare interface Prompt {
    type: typeof Prompt;
    source?: string;
    variable: Variable | null;
}

/**
 * @alpha
 */
declare const RepeatLoop = "RepeatLoop";

/**
 * @alpha
 */
declare interface RepeatLoop {
    type: typeof RepeatLoop;
    source?: string;
    value: ValueExpression | null;
    extra: boolean | null;
}

/**
 * @alpha
 */
declare const ReturnStatement = "ReturnStatement";

/**
 * @alpha
 */
declare interface ReturnStatement {
    type: typeof ReturnStatement;
    source?: string;
}

/**
 * @alpha
 */
export declare function run(lines: types.Statement[], options?: RunOptions): ProgramHandle;

/**
 * @alpha
 */
export declare interface RunOptions {
    source?: string | string[];
    frequencyMs?: number;
    outputCallback?: (value: string, newline: boolean) => void;
    elem?: JQuery;
    debug?: boolean;
    callback?: (status: string) => void;
    input?: JQuery;
    stdin?: string;
    includeErrors?: boolean;
    includeLibErrors?: boolean;
    includeLineNumbers?: boolean;
    includeSource?: boolean;
}

/**
 * @alpha
 */
declare const Send = "Send";

/**
 * @alpha
 */
declare interface Send {
    type: typeof Send;
    source?: string;
    variable: Variable | null;
}

/**
 * @alpha
 */
declare type Statement = EmptyStatement | SyntaxError_2 | AssignmentStatement | CtlStatement | IoStatement | ValueStatement;

/**
 * @alpha
 */
declare const StopStatement = "StopStatement";

/**
 * @alpha
 */
declare interface StopStatement {
    type: typeof StopStatement;
    source?: string;
}

/**
 * @alpha
 */
declare const STRING = "string";

/**
 * @alpha
 */
declare const STRINGVARIABLE = "stringVariable";

/**
 * @alpha
 */
declare interface StringVariable {
    type: 'stringVariable';
    name: string;
}

/**
 * @alpha
 */
declare const SyntaxError_2 = "SyntaxError";

/**
 * @alpha
 */
declare interface SyntaxError_2 {
    type: typeof SyntaxError_2;
    source?: string;
}

/**
 * @alpha
 */
declare const ThenStatement = "ThenStatement";

/**
 * @alpha
 */
declare interface ThenStatement {
    type: typeof ThenStatement;
    source?: string;
    extra: boolean | null;
}

/**
 * @alpha
 */
declare const ti = "ti";

/**
 * @alpha
 */
declare type TiList = ListLiteral | ListResolved;

/**
 * @alpha
 */
declare type TiNumber = NumberLiteral | NumberResolved;

/**
 * @alpha
 */
declare interface TiString {
    type: typeof STRING;
    chars: string;
}

declare namespace types {
    export {
        ti,
        lib,
        SyntaxError_2 as SyntaxError,
        NUMBER,
        LIST,
        STRING,
        ListVariablePrefix,
        NumberLiteral,
        NumberResolved,
        TiNumber,
        TiString,
        ListLiteral,
        ListResolved,
        TiList,
        ValueResolved,
        ANS,
        GetKey,
        Ans,
        VARIABLE,
        STRINGVARIABLE,
        LISTVARIABLE,
        LISTINDEX,
        NumericVariable,
        StringVariable,
        ListVariable,
        ListIndex,
        Variable,
        Assignable,
        BINARY,
        UNARY,
        BinaryExpression,
        UnaryExpression,
        ValueExpression,
        EmptyStatement,
        ValueStatement,
        AssignmentStatement,
        IfStatement,
        ThenStatement,
        ElseStatement,
        ForLoop,
        WhileLoop,
        RepeatLoop,
        EndStatement,
        PauseStatement,
        LabelStatement,
        GotoStatement,
        IncrementSkip,
        DecrementSkip,
        MenuStatement,
        ProgramStatement,
        ReturnStatement,
        StopStatement,
        DelVarStatement,
        GraphStyleStatement,
        OpenLibStatement,
        ExecLibStatement,
        MenuChoice,
        CtlStatement,
        Input,
        Prompt,
        Display,
        DispGraph,
        DispTable,
        Output,
        ClrHome,
        ClrTable,
        GetCalc,
        Get,
        Send,
        IoStatement,
        Statement
    }
}
export { types }

/**
 * @alpha
 */
declare const UNARY = "unary";

/**
 * @alpha
 */
declare interface UnaryExpression {
    type: typeof UNARY;
    operator: string;
    argument: ValueExpression;
}

/**
 * @alpha
 */
declare type ValueExpression = TiNumber | TiString | TiList | Assignable | BinaryExpression | UnaryExpression | SyntaxError_2 | Ans | GetKey;

/**
 * @alpha
 */
declare type ValueResolved = NumberResolved | TiString | ListResolved;

/**
 * @alpha
 */
declare const ValueStatement = "value";

/**
 * @alpha
 */
declare interface ValueStatement {
    type: typeof ValueStatement;
    value: ValueExpression;
    source?: string;
}

/**
 * @alpha
 */
declare const VARIABLE = "variable";

/**
 * @alpha
 */
declare type Variable = NumericVariable | StringVariable | ListVariable;

/**
 * @alpha
 */
declare const WhileLoop = "WhileLoop";

/**
 * @alpha
 */
declare interface WhileLoop {
    type: typeof WhileLoop;
    source?: string;
    value: ValueExpression | null;
    extra: boolean | null;
}

export { }
